﻿Imports System.Runtime.CompilerServices

Namespace App
    Public Module App

        Public Icon As Icon = My.Resources.Icon_16x16
        Public _tabControl As TabControl
        Public _mdiParent As Form

    End Module

End Namespace

Public Module TabbedMdi

    ''' <summary> Anzeigen einer ChildForm - Optional mit ControlStyling und Registrierung des DataSets, sowie verstecken des TreeView nach Öffnen der Form und App-Icon</summary>
    <Extension>
    Public Sub ShowMdiChild(Child As Form)
        RegisterMdiHandle() ' Registrierung der Handle-Methoden
        With Child
            If .MdiParent Is App._mdiParent Then
                .Select()
            Else
                With App._tabControl
                    .SizeMode = TabSizeMode.Fixed
                    .ItemSize = New Size(0, 1)
                End With
                App._mdiParent.SuspendLayout()
                Dts.Register(.Self, False) ' Optional nur mit den Helpers von ErfinderDesRades: Registriert das DataSet auf der Form
                ControlStyling.StyleControls(.Self)
                .WindowState = FormWindowState.Maximized ' Je nach Anwendungsfall entweder alle Childs maximiert öffnen, oder nur das 1.
#If False Then
                If _mdiParent.MdiChildren.Count = 0 Then .WindowState = FormWindowState.Maximized
#End If
                '.Icon = App.Icon
                .ShowIcon = False
                .ControlBox = False
                .MdiParent = App._mdiParent
                .MinimizeBox = False 'Minimize-Boxen auf Child-Forms finde ich unnötig
                .KeyPreview = True
                .Show()
                'idiotischerweise muss der ChildForm-Menustrip, dessen Items einerseits automatisch in den Parent-Menustrip
                ' eingemerged werden, andererseits explizit versteckt werden - sonst verbleibt ein leerer Menustrip auffm ChildForm
                Child.Controls.OfType(Of MenuStrip)().SingleOrDefault()?.Hide()
                App._mdiParent.ResumeLayout()
            End If
        End With
    End Sub
    '    Public Sub ShowMdiChild(Child As Form)
    '        RegisterMdiHandle() ' Registrierung der Handle-Methoden
    '        With Child
    '            If .MdiParent Is App._mdiParent Then
    '                .Select()
    '            Else
    '                App._mdiParent.SuspendLayout()
    '                Dts.Register(.Self, False) ' Optional nur mit den Helpers von ErfinderDesRades: Registriert das DataSet auf der Form
    '                ControlStyling.StyleControls(.Self)
    '                .WindowState = FormWindowState.Maximized ' Je nach Anwendungsfall entweder alle Childs maximiert öffnen, oder nur das 1.
    '#If False Then
    '                If _mdiParent.MdiChildren.Count = 0 Then .WindowState = FormWindowState.Maximized
    '#End If
    '                .Icon = App.Icon
    '                .MdiParent = App._mdiParent
    '                .MinimizeBox = False 'Minimize-Boxen auf Child-Forms finde ich unnötig
    '                .KeyPreview = True
    '                .Show()
    '                'idiotischerweise muss der ChildForm-Menustrip, dessen Items einerseits automatisch in den Parent-Menustrip
    '                ' eingemerged werden, andererseits explizit versteckt werden - sonst verbleibt ein leerer Menustrip auffm ChildForm
    '                Child.Controls.OfType(Of MenuStrip)().SingleOrDefault()?.Hide()
    '                App._mdiParent.ResumeLayout()
    '            End If
    '        End With
    '    End Sub

    ''' <summary> Registriert die Handle-Methoden für die Steuerung der Child-Forms </summary>
    Private Sub RegisterMdiHandle()
        RemoveHandler App._mdiParent.MdiChildActivate, AddressOf MdiChildOpen
        RemoveHandler App._tabControl.SelectedIndexChanged, AddressOf Tab_SelectedIndexChanged
        AddHandler App._mdiParent.MdiChildActivate, AddressOf MdiChildOpen
        AddHandler App._tabControl.SelectedIndexChanged, AddressOf Tab_SelectedIndexChanged
    End Sub

    ''' <summary> Stellt beim Schließen des Childs sicher, dass die entsprechende TabPage entfernt wird </summary>
    Private Sub MdiChildClose(sender As Object, e As FormClosedEventArgs)
        DirectCast(DirectCast(sender, Form).Tag, TabPage).Dispose()
    End Sub

    ''' <summary> Prüft, ob das Child bereits existiert, falls nicht wird eine neue TabPage erstellt </summary>
    Private Sub MdiChildOpen(sender As Object, e As EventArgs)
        If App._mdiParent.ActiveMdiChild Is Nothing Then
            App._tabControl.Visible = False
        Else
            Dim tp = DirectCast(App._mdiParent.ActiveMdiChild.Tag, TabPage)
            If tp Is Nothing Then
                tp = New TabPage(App._mdiParent.ActiveMdiChild.Text)
                tp.Tag = App._mdiParent.ActiveMdiChild
                App._mdiParent.ActiveMdiChild.Tag = tp
                App._tabControl.TabPages.Add(tp)
                AddHandler App._mdiParent.ActiveMdiChild.FormClosed, AddressOf MdiChildClose
                If Not App._tabControl.Visible Then App._tabControl.Visible = True
            End If
            App._tabControl.SelectedTab = tp
        End If
    End Sub

    ''' <summary> Ruft beim wechseln der TabPage das entsprechende Child auf und zeigt es an</summary>
    Private Sub Tab_SelectedIndexChanged(sender As Object, e As EventArgs)
        Dts.SaveDts() 'vor dem Wechsel der ChildForms DataSet speichern, damit nix verloren geht
        DirectCast(App._tabControl.SelectedTab?.Tag, Form)?.Select()
    End Sub

End Module

Public Module LocalHelper

    ''' <summary>sendet TAB</summary>
    Public Sub sendTab()
        SendKeys.SendWait("{TAB}")
    End Sub

End Module

Public Module MessageBoxes

    ''' <summary>zeigt eine Messagebox, wenn die gefilterte Bindingsource keine Elemente enthält</summary>
    <Extension>
    Public Sub msgDGVnoEntry(bs As BindingSource)
        If bs.Count = 0 Then msgInformation("keine Einträge")
    End Sub

    Public Sub msgDeleteDenied()
        msgInformation("Löschen nicht möglich, bitte auf inaktiv umstellen")
    End Sub

    Public Function msgOkCancel(Nachricht As String, Titel As String) As DialogResult
        Return MessageBox.Show(Nachricht, Titel, MessageBoxButtons.OKCancel, MessageBoxIcon.Information, Nothing) ', MessageBoxOptions.ServiceNotification)
    End Function
    Public Sub msgInformation(Nachricht As String)
        MessageBox.Show(Nachricht, "Information", MessageBoxButtons.OK, MessageBoxIcon.Information, Nothing) ', MessageBoxOptions.ServiceNotification)
    End Sub

    Public Sub Msg(ParamArray items As Object())
        MessageBox.Show(String.Concat(items))
    End Sub

    Public Sub msgEx(ex As Exception, Optional customInfo As String = Nothing)
        If customInfo Is Nothing Then
            MessageBox.Show(ex.Message, "Ausnahmefehler", MessageBoxButtons.OK, MessageBoxIcon.Error, Nothing) ', MessageBoxOptions.ServiceNotification)
        Else
            MessageBox.Show($"{ex.Message}{Environment.NewLine}{customInfo}", "Ausnahmefehler", MessageBoxButtons.OK, MessageBoxIcon.Error, Nothing) ', MessageBoxOptions.ServiceNotification)
        End If
    End Sub

    Public Sub msgExclamation(Nachricht As String)
        MessageBox.Show(Nachricht, "Wichtige Info", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, Nothing) ', MessageBoxOptions.ServiceNotification)
    End Sub

    Public Sub msgError(Nachricht As String)
        MessageBox.Show(Nachricht, "FEHLER", MessageBoxButtons.OK, MessageBoxIcon.Error, Nothing) ', MessageBoxOptions.ServiceNotification)
    End Sub

    Public Function msgQuestionWarning(Nachricht As String) As DialogResult
        Return MessageBox.Show(Nachricht, "Warnung", MessageBoxButtons.YesNo, MessageBoxIcon.Warning, Nothing) ', MessageBoxOptions.ServiceNotification)
    End Function

    Public Function msgQuestionInfo(Nachricht As String) As DialogResult
        Return MessageBox.Show(Nachricht, "Frage", MessageBoxButtons.YesNo, MessageBoxIcon.Question, Nothing) ', MessageBoxOptions.ServiceNotification)
    End Function

End Module
